// ----------------------------------------------------------------------------
// ClassicalProblem.java
// A proving problem in classical logic.
// $Id: ClassicalProblem.java,v 1.3 2010/03/03 14:01:27 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009-, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.ewuni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Proving;

import java.io.*;

import fmrisc.ProgramExplorer.Syntax.*;
import fmrisc.ProgramExplorer.InOut.*;
import fmrisc.ProgramExplorer.Syntax.Logic.*;

/**
 * 
 * A proving problem in classical logic.
 *
 */
public class ClassicalProblem
{
  private Declaration[] decls; // the declarations
  private Formula goal;        // the goal to be proved
  private ErrorStream out;     // stream for printing messages
  
  /****************************************************************************
   * Construct a proving problem.
   * @param decls the declarations of entities refered in the goal.
   * @param goal the goal to be proved.
   * @param out a stream for printing messages.
   ***************************************************************************/
  public ClassicalProblem(Declaration[] decls, Formula goal, ErrorStream out)
  {
    this.decls = decls;
    this.goal = goal;
    this.out = out;
  }
  
  /***************************************************************************
   * Print task to output stream.
   * @param out the stream to print to
   **************************************************************************/
  public void print(PrintWriter out)
  {
    out.println("Declarations:");
    for (Declaration d : decls)
    {
      d.print(out);
      out.println(";");
    }
    out.print("Goal: ");
    goal.print(out);
    out.println("");
  }
  
  /***************************************************************************
   * Get the declarations. 
   * @return the declarations.
   **************************************************************************/
  public Declaration[] getDeclarations()
  {
    return decls;
  }
  
  /***************************************************************************
   * Get the goal.
   * @return the goal.
   **************************************************************************/
  public Formula getGoal()
  {
    return goal;
  }
  
  
  /****************************************************************************
   * Translate classical problem to RISC ProofNavigator Problem
   * @param dir the directory to which information may be written
   * @return the translation.
   **************************************************************************/
  public ProofNavigatorProblem proofNavigatorProblem(File dir)
  {
    return ProofNavigator.translate(this, dir, out);
  }
}
