// ----------------------------------------------------------------------------
// ReferenceCloner.java
// Clone an expression transforming all references to certain symbols.
// $Id: ReferenceCloner.java,v 1.11 2011/09/30 16:02:16 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Semantics;

import java.util.*;

import fmrisc.ProgramExplorer.*;
import fmrisc.ProgramExplorer.Syntax.*;
import fmrisc.ProgramExplorer.Syntax.Logic.*;

/**
 * 
 * Clone an expression transforming all references to certain symbols.
 *
 */
public class ReferenceCloner extends ASTCloner
{
  /****************************************************************************
   * Create a clone (deep copy) of a tree replacing
   * references to certain symbols.
   * @param tree the tree to be cloned 
   * @param map a map of (original) symbols to (new) symbols.
   * @return the clone of the tree 
   ***************************************************************************/
  public static AST clone(AST tree, Map<Symbol,Symbol> map)
  {
    ReferenceCloner cloner = new ReferenceCloner(map);
    return cloner.clone0(tree);
  }
  
  // the map of (original) symbols to (new) symbols.
  Map<Symbol,Symbol> map;
  
  /****************************************************************************
   * Create a cloner for the denoted map.
   * @param map a map of (original) symbols to (new) symbols.
   ***************************************************************************/
  protected ReferenceCloner(Map<Symbol,Symbol> map)
  {
    this.map = map;
  }
  
  /****************************************************************************
   * Replace reference to symbol by new reference name.
   * @param ref the reference to be replaced.
   ***************************************************************************/
  public void visit(Reference ref) 
  { 
    Symbol symbol = ref.getName().getBase().getSymbol();
    if (symbol == null)
      Main.internalError("null symbol: " + ref);
    Symbol symbol1 = map.get(symbol);
    if (symbol1 == null) 
    {
      super.visit(ref);
      return;
    }
    Identifier ident0 = symbol1.getName();
    Identifier ident1 = Identifier.construct(ident0.getString());
    ident1.setSymbol(symbol1);
    result = Reference.construct(ident1);
    if (symbol1 instanceof ValueSymbol)
    {
      ValueSymbol symbol2 = (ValueSymbol)symbol1;
      ((Term)result).setType(symbol2.getType());
    }
  }
}

