// ----------------------------------------------------------------------------
// ReferencedVars.java
// Compute the program variables referenced by an expression.
// $Id: ReferencedVars.java,v 1.11 2011/09/30 16:02:16 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Semantics;

import fmrisc.ProgramExplorer.*;
import fmrisc.ProgramExplorer.Syntax.*;
import fmrisc.ProgramExplorer.Syntax.Logic.*;

import java.util.*;

/**
 * 
 * Compute the program variables referenced by an expression.
 *
 */
public final class ReferencedVars extends ASTVisitorBase
{
  /***************************************************************************
   * Compute referenced variables
   * @param tree the expression.
   * @return a collection of the symbols of the variables referenced by tree. 
   **************************************************************************/
  public static Collection<VariableSymbol> compute(AST tree)
  {
    ReferencedVars ref = new ReferencedVars();
    tree.accept(ref);
    return ref.symbols;
  }
  
  /***************************************************************************
   * Add references from the denoted tree.
   * @param tree the tree.
   **************************************************************************/
  public void add(AST tree)
  {
    tree.accept(this);
  }
  
  /***************************************************************************
   * Get the result of the computation.
   * @return the result.
   **************************************************************************/
  public Collection<VariableSymbol> getResult()
  {
    return symbols;
  }
  
  // the collection of symbols computed up to now
  private Collection<VariableSymbol> symbols = new SetClass<VariableSymbol>();
  
  /****************************************************************************
   * Process variable.
   * @param var a program variable.
   ***************************************************************************/
  public void visit(ProgramVariableBase var) 
  { 
    Identifier[] idents = var.getName().idents;
    for (Identifier ident : idents)
    {
      Symbol symbol = ident.getSymbol();
      if (!(symbol instanceof VariableSymbol)) continue;
      VariableSymbol symbol0 = (VariableSymbol)symbol;
      symbols.add(symbol0);
    }
  }
 
}


