// ----------------------------------------------------------------------------
// VariableSymbolBase.java
// A symbol denoting a variable.
// $Id: VariableSymbolBase.java,v 1.10 2011/05/18 14:05:53 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Semantics;

import java.util.*;

import fmrisc.ProgramExplorer.*;
import fmrisc.ProgramExplorer.Syntax.*;
import fmrisc.ProgramExplorer.Syntax.Program.*;
import fmrisc.ProgramExplorer.Syntax.Logic.Type;

/**
 * 
 * A symbol denoting a variable.
 *
 */
public abstract class VariableSymbolBase 
extends SymbolBase implements VariableSymbol
{
  private final TypeExpression type;   // the type of the symbol
  private Type ltype;                  // the logical type (may be null)
  
  /***************************************************************************
   * Construct variable symbol from name and type.
   * @param name the name of the symbol.
   * @param type the type of the symbol.
   **************************************************************************/
  protected VariableSymbolBase(Identifier name, TypeExpression type)
  {
    super(name);
    this.type = type;
  }
  
  /***************************************************************************
   * Get type of symbol.
   * @return the symbol's type
   **************************************************************************/
  public TypeExpression getType()
  {
    return type;
  }
  
  /***************************************************************************
   * Set logic type of symbol.
   * @param type the symbol's logic type
   **************************************************************************/
  public void setLogicType(Type type)
  {
    if (type == null) Main.internalError("no logic type: " + getDeclaration());
    this.ltype = type;
  }
  
  /***************************************************************************
   * Get logic type of symbol.
   * @return the symbol's logic type
   **************************************************************************/
  public Type getLogicType()
  {
    if (ltype == null) Main.internalError("no logic type: " + getDeclaration());
    return ltype;
  }
  
  /***************************************************************************
   * Apply map to set of variable symbols.
   * @param map a mapping of symbols to symbols.
   * @param symbols a set of variable symbols.
   * @return the result of applying the map to the symbol set
   *         (if a symbol is not in the domain of the map, it remains unchanged)
   **************************************************************************/
  public static Set<VariableSymbol> apply(Map<Symbol, Symbol> map, 
      Set<VariableSymbol> symbols)
  {
    Set<VariableSymbol> result = new SetClass<VariableSymbol>();
    for (VariableSymbol symbol : symbols)
    {
      VariableSymbol symbol0 = (VariableSymbol)map.get(symbol);
      if (symbol0 == null)
        symbol0 = symbol;
      else
        System.out.println(">>>>>>>>>>>>>>>>" + symbol0);
      result.add(symbol0);
    }
    return result;
  }
}
