// ----------------------------------------------------------------------------
// VariableTypes.java
// Set logical types of all variables declared in program.
// $Id: VariableTypes.java,v 1.5 2011/09/30 15:47:29 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Semantics;

import fmrisc.ProgramExplorer.Syntax.*;
import fmrisc.ProgramExplorer.Syntax.Program.*;
import fmrisc.ProgramExplorer.Syntax.Logic.*;
/**
 * 
 * Set logical types of all variables declared in program.
 *
 */
public class VariableTypes extends ASTVisitorBase
{
  /***************************************************************************
   * Set the logical types of all variable symbols.
   * @param tree a program tree.
   * @param trans a type translator.
   **************************************************************************/
  public static void process(AST tree, TypeTranslator trans)
  {
    VariableTypes visitor = new VariableTypes(trans);
    tree.accept(visitor);
  }
  
  private final TypeTranslator trans;
  
  /****************************************************************************
   * Create a variable processor with the denoted type translator.
   * @param trans the type translator.
   ***************************************************************************/
  private VariableTypes(TypeTranslator trans)
  {
    this.trans = trans;
  }
  
  public void visit(DeclCallStatement decl)
  {
    setType(decl);
  }
  
  public void visit(GlobalVariableDeclaration decl)
  {
    setType(decl);
  }
  
  public void visit(VariableDeclarationBase decl)
  {
    setType(decl);
  }
  
  /***************************************************************************
   * Set logical type of declared variable symbol.
   * @param decl the declaration of the variable.
   **************************************************************************/
  private void setType(VariableDeclaration decl)
  {
    VariableSymbol symbol = (VariableSymbol)decl.getName().getSymbol();
    Type type = trans.logicType(symbol.getType());
    symbol.setLogicType(type);
  }
}
