// ----------------------------------------------------------------------------
// ASTVisitor.java
// Interface to abstract syntax tree visitors.
// $Id: ASTVisitor.java,v 1.8 2012/10/18 16:12:57 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax;

import fmrisc.ProgramExplorer.Syntax.Logic.*;
import fmrisc.ProgramExplorer.Syntax.Program.*;
import fmrisc.ProgramExplorer.Syntax.Spec.*;
import fmrisc.ProgramExplorer.Judgements.*;

/**
 * Interface to abstract syntax tree visitors.
 */
public interface ASTVisitor
{
  /****************************************************************************
   * Visit an abstract syntax tree.
   * @param tree the node of the abstract syntax tree that is visited.
   ***************************************************************************/
  public void visit(AST tree);
  
  // more specific calls below
  // allows accept() to select concrete visit() rather than generic above()
  public void visit(AndExpression tree);
  public void visit(AndFormula tree);
  public void visit(ApplicationExpression tree);
  public void visit(ArrayTerm tree);
  public void visit(AssertionStatement tree);
  public void visit(AssignCallStatement tree);
  public void visit(AssignmentStatement tree);
  public void visit(AssignNewStatement tree);
  public void visit(ASTLogicBase tree);
  public void visit(ASTProgramAnnotatedBase tree);
  public void visit(ASTProgramBase tree);
  public void visit(ASTSourceBase tree);
  public void visit(AtomicFormula tree);
  public void visit(AtomicType tree);
  public void visit(AxiomDefinition tree);
  public void visit(BinaryAtomicFormula tree);
  public void visit(BinaryExpression tree);
  public void visit(BinaryFormula tree);
  public void visit(BinaryTerm tree);
  public void visit(BinaryValueExpression tree);
  public void visit(BitAndTerm tree);
  public void visit(BitFalse tree);
  public void visit(BitLiteral tree);
  public void visit(BitNegationTerm tree);
  public void visit(BitOrTerm tree);
  public void visit(BitTrue tree);
  public void visit(BitType tree);
  public void visit(BlockStatement tree);
  public void visit(BreakStatement tree);
  public void visit(CallStatement tree);
  public void visit(CharLiteral tree);
  public void visit(CharType tree);
  public void visit(ClassDeclaration tree);
  public void visit(ClassImport tree);
  public void visit(ClassMethodDeclaration tree);
  public void visit(ClassSpec tree);
  public void visit(ClassVariableDeclaration tree);
  public void visit(CompilationUnitBase tree);
  public void visit(ConditionalStatement tree);
  public void visit(Constant tree);
  public void visit(ConstructorDeclaration tree);
  public void visit(ContinueStatement tree);
  public void visit(DeclCallStatement tree);
  public void visit(DeclNewStatement tree);
  public void visit(DefinitionBase tree);
  public void visit(DividesExpression tree);
  public void visit(DividesTerm tree);
  public void visit(EmptyStatement tree);
  public void visit(EqualsExpression tree);
  public void visit(EqualsFormula tree);
  public void visit(EquivalentFormula tree);
  public void visit(ExceptionType tree);
  public void visit(ExistsFormula tree);
  public void visit(ExpressionBase tree);
  public void visit(False tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Logic.ArrayType tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Logic.BooleanLiteral tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Logic.BooleanType tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Logic.DeclarationBase tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Logic.IntLiteral tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Logic.IntType tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Logic.SelectorBase tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Logic.StringLiteral tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Logic.StringType tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Program.ArrayType tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Program.BooleanLiteral tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Program.BooleanType tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Program.DeclarationBase tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Program.IntLiteral tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Program.IntType tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Program.SelectorBase tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Program.StringLiteral tree);
  public void visit(fmrisc.ProgramExplorer.Syntax.Program.StringType tree);
  public void visit(ForallFormula tree);
  public void visit(LoopStatementBase tree);
  public void visit(ForLoopStatement tree);
  public void visit(FormulaBase tree);
  public void visit(FormulaDefinition tree);
  public void visit(FunctionApplication tree);
  public void visit(FunctionType tree);
  public void visit(GlobalVariableDeclaration tree);
  public void visit(GreaterEqualExpression tree);
  public void visit(GreaterEqualFormula tree);
  public void visit(GreaterExpression tree);
  public void visit(GreaterFormula tree);
  public void visit(Handler tree);
  public void visit(Identifier tree);
  public void visit(IdentifierSelector tree);
  public void visit(IfThenElseExpression tree);
  public void visit(IfThenElseFormula tree);
  public void visit(IfThenElseTerm tree);
  public void visit(ImpliesFormula tree);
  public void visit(ImportBase tree);
  public void visit(IndexSelector tree);
  public void visit(LambdaFormula tree);
  public void visit(LambdaTerm tree);
  public void visit(LeftValue tree);
  public void visit(LessEqualExpression tree);
  public void visit(LessEqualFormula tree);
  public void visit(LessExpression tree);
  public void visit(LessFormula tree);
  public void visit(LetExpression tree);
  public void visit(LetFormula tree);
  public void visit(LetTerm tree);
  public void visit(LoopSpec tree);
  public void visit(MessageSelector tree);
  public void visit(MethodDeclaration tree);
  public void visit(MethodSpec tree);
  public void visit(MinusExpression tree);
  public void visit(MinusTerm tree);
  public void visit(Name tree);
  public void visit(NamedType tree);
  public void visit(NatType tree);
  public void visit(NegationExpression tree);
  public void visit(NegationTerm tree);
  public void visit(NewArrayExpression tree);
  public void visit(NewState tree);
  public void visit(NewType tree);
  public void visit(NotEqualsExpression tree);
  public void visit(NotEqualsFormula tree);
  public void visit(SimilarFormula tree);
  public void visit(NotEquivalentFormula tree);
  public void visit(NotExpression tree);
  public void visit(NotFormula tree);
  public void visit(NullLiteral tree);
  public void visit(NullType tree);
  public void visit(ObjectMethodDeclaration tree);
  public void visit(ObjectVariableDeclaration tree);
  public void visit(OldState tree);
  public void visit(OrExpression tree);
  public void visit(OrFormula tree);
  public void visit(PackageImport tree);
  public void visit(ParamDeclarationBase tree);
  public void visit(Parameter tree);
  public void visit(PercentExpression tree);
  public void visit(PlusExpression tree);
  public void visit(PlusTerm tree);
  public void visit(PostfixApplication tree);
  public void visit(PostfixFormula tree);
  public void visit(PostfixTerm tree);
  public void visit(PostVariable tree);
  public void visit(PreVariable tree);
  public void visit(PrimitiveType tree);
  public void visit(ProgramAnnotationBase tree);
  public void visit(ProgramVariableBase tree);
  public void visit(QuantifiedExpression tree);
  public void visit(QuantifiedFormula tree);
  public void visit(QuantifiedTerm tree);
  public void visit(ReadsOnlyFormula tree);
  public void visit(Real0Type tree);
  public void visit(RealType tree);
  public void visit(RecordTerm tree);
  public void visit(RecordType tree);
  public void visit(Reference tree);
  public void visit(ReturnStatement tree);
  public void visit(SelectionTerm tree);
  public void visit(SelectorExpression tree);
  public void visit(SelectorIdentifier tree);
  public void visit(SelectorIndex tree);
  public void visit(SelectorNumber tree);
  public void visit(SpecificationBase tree);
  public void visit(StateBreaks tree);
  public void visit(StateContinues tree);
  public void visit(StateExecutes tree);
  public void visit(StateLiteral tree);
  public void visit(StatementBase tree);
  public void visit(StatementSpec tree);
  public void visit(StateMessage tree);
  public void visit(StateReturns tree);
  public void visit(StateThrows tree);
  public void visit(StateThrowsException tree);
  public void visit(StateType tree);
  public void visit(StateValue tree);
  public void visit(StatJudgement tree);
  public void visit(SubrangeType tree);
  public void visit(SubType tree);
  public void visit(TermApplication tree);
  public void visit(TermBase tree);
  public void visit(TheoryDeclaration tree);
  public void visit(ThrowStatement tree);
  public void visit(TimesExpression tree);
  public void visit(TimesTerm tree);
  public void visit(TopDeclarationBase tree);
  public void visit(True tree);
  public void visit(TryCatchStatement tree);
  public void visit(TupleTerm tree);
  public void visit(TupleType tree);
  public void visit(TypeBase tree);
  public void visit(TypeDeclaration tree);
  public void visit(TypeDefinition tree);
  public void visit(TypedIdentifier tree);
  public void visit(UnaryExpression tree);
  public void visit(UnaryFormula tree);
  public void visit(UnaryTerm tree);
  public void visit(UnaryValueExpression tree);
  public void visit(UnitSpec tree);
  public void visit(UpdateTerm tree);
  public void visit(ValueDeclarationClass tree);
  public void visit(ValueDeclarationFormula tree);
  public void visit(ValueDeclarationInitialized tree);
  public void visit(ValueDefinitionClass tree);
  public void visit(ValuedIdentifier tree);
  public void visit(ValueExpressionBase tree);
  public void visit(VariableDeclarationBase tree);
  public void visit(VariableStatement tree);
  public void visit(VariableValue tree);
  public void visit(VoidCallStatement tree);
  public void visit(VoidType tree);
  public void visit(WhileLoopStatement tree);
  public void visit(WritesOnlyFormula tree); 
}
