// ----------------------------------------------------------------------------
// ArrayType.java
// An array type.
// $Id: ArrayType.java,v 1.7 2017/07/04 08:04:38 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the types of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * An array type.
 *
 */
public final class ArrayType extends TypeBase
{
  public final static String arrayKeyword = "ARRAY";
  public final static String baseKeyword = "OF";
  
  private static final int precedence = 0; // lowest precedence, see SpecLang.g
  private final Type index;
  private final Type base;
  
  /****************************************************************************
   * Construct array type.
   * @param index the index type.
   * @param base the base type.
   * @return the type.
   ***************************************************************************/
  public static ArrayType construct(Type index, Type base)
  {
    return new ArrayType(index, base);
  }
  
  /****************************************************************************
   * Construct array type.
   * @param index the index type.
   * @param base the base type.
   ***************************************************************************/
  private ArrayType(Type index, Type base)
  {
    this.index = index;
    this.base = base;
  }
  
  /****************************************************************************
   * Get index type.
   * @return the index type.
   ***************************************************************************/
  public Type getIndex()
  {
    return index;
  }
  
  /****************************************************************************
   * Get base type.
   * @return the base type.
   ***************************************************************************/
  public Type getBase()
  {
    return base;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
    index.accept(visitor);
    base.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    index.accept(visitor);
    base.accept(visitor);
  }
  
  /***************************************************************************
   * Get precedence value of the type operator 
   * (higher precedence value means higher binding power)
   **************************************************************************/
  public int getPrecedence()
  {
    return precedence;
  }
}
