// ----------------------------------------------------------------------------
// DeclarationBase.java
// The base class of declarations.
// $Id: DeclarationBase.java,v 1.5 2011/09/30 15:47:33 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.InOut.SourcePosition;
import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * The base class of declarations.
 * 
 */
public abstract class DeclarationBase extends ASTLogicBase 
  implements Declaration
{
  protected final Identifier name;
  private SourcePosition position = null;
  
  /****************************************************************************
   * Construct declaration with denoted name.
   * @param name the name of the declared entity.
   ***************************************************************************/
  protected DeclarationBase(Identifier name)
  {
    this.name = name;
  }
  
  /****************************************************************************
   * Get the name of the declared entity.
   * @return the entity's name.
   ***************************************************************************/
  public Identifier getName()
  {
    return name;
  }
  
  /***************************************************************************
   * Set the position of this node in the source code.
   * @param position the source code position.
   **************************************************************************/
  public void setPosition(SourcePosition position)
  {
    this.position = position;
  }
  
  /***************************************************************************
   * Get the position of this node in the source code (may be null).
   * @return the source code position.
   **************************************************************************/
  public SourcePosition getPosition()
  {
    return position;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    name.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    name.accept(visitor);
  } 
}

