// ----------------------------------------------------------------------------
// Expression.java
// Interface to any kind of logical expression.
// $Id: Expression.java,v 1.6 2017/07/04 08:04:38 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * Interface to any kind of logical expression.
 *
 */
public interface Expression extends ASTLogic, ASTSource
{  
  /***************************************************************************
   * Get precedence value of the expression operator 
   * (higher precedence value means higher binding power)
   * @return the precedence value
   **************************************************************************/
  public int getPrecedence();
  
  /***************************************************************************
   * True if "(e1 op e2) op e3" can be printed as "e1 op e2 op e3"
   * @return true if above printing is possible.
   **************************************************************************/
  public boolean isLeftAssociative();
  
  /***************************************************************************
   * True if "e1 op (e2 op e3)" can be printed as "e1 op e2 op e3"
   * @return true if above printing is possible.
   **************************************************************************/
  public boolean isRightAssociative();
}
