// ----------------------------------------------------------------------------
// StateMessage.java
// The exception message captured by a state.
// $Id: StateMessage.java,v 1.6 2017/07/04 08:04:38 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * The exception message captured by a state.
 *
 */
public final class StateMessage extends PostfixTerm
{
  private static final int precedence = 70; // see SpecLang.g
  
  /****************************************************************************
   * The exception message captured by a state.
   * @param arg the argument denoting the state.
   * @return the message.
   ***************************************************************************/
  public static StateMessage construct(Term arg)
  {
    return new StateMessage(arg);
  }
  
  /****************************************************************************
   * The exception message captured by a state.
   * @param arg the argument denoting the state.
   ***************************************************************************/
  private StateMessage(Term arg)
  {
    super(arg, Identifier.construct("MESSAGE"));
  }
  
  /***************************************************************************
   * Get precedence value of the expression operator 
   * (higher precedence value means higher binding power)
   **************************************************************************/
  public int getPrecedence()
  {
    return precedence;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
}
