// ----------------------------------------------------------------------------
// TypeDeclaration.java
// The declaration of a type.
// $Id: TypeDeclaration.java,v 1.4 2011/09/30 15:47:33 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Semantics.*;
import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * The declaration of a type.
 *
 */
public class TypeDeclaration extends DeclarationBase
{
  public final static String keyword = "TYPE";
  
  /****************************************************************************
   * Create new declaration of a constant denoting a type.
   * @param name the name of the constant.
   * @return the declaration
   ***************************************************************************/
  public static TypeDeclaration construct(Identifier name)
  {
    return new TypeDeclaration(name);
  }
    
  /****************************************************************************
   * Create new declaration of a constant denoting a type.
   * @param name the name of the constant.
   * @return the declaration
   ***************************************************************************/
  protected TypeDeclaration(Identifier name)
  {
    super(name);
  }
  
  /***************************************************************************
   * Get symbol introduced by declaration.
   * @return the symbol.
   **************************************************************************/
  public final TypeSymbol getSymbol()
  {
    return (TypeSymbol)name.getSymbol();
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
    name.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
}
