// ----------------------------------------------------------------------------
// AssignCallStatement.java
// A method call statement with a result value assigned to a variable.
// $Id: AssignCallStatement.java,v 1.5 2017/07/04 08:04:37 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Program;

import java.util.*;
import fmrisc.ProgramExplorer.Syntax.*;

/**
 * A method call statement with a result value assigned to a variable.
 */
public class AssignCallStatement extends CallStatement
{
  public final LeftValue lval;
  
  /****************************************************************************
   * Construct a call statement with a result value assigned to a variable.
   * @param lval the left value taking the result (may be null)
   * @param method the method name
   * @param args the method arguments
   * @return the statement.
   ***************************************************************************/
  public static AssignCallStatement construct
    (LeftValue lval, Name method, Vector<ValueExpression> args)
  {
    return new AssignCallStatement(lval, method, 
        args.toArray(new ValueExpression[args.size()]));
  }
  
  /****************************************************************************
   * Construct a call statement with a result value assigned to a variable.
   * @param lval the left value taking the result (may be null)
   * @param method the method name
   * @param args the method arguments
   * @return the statement.
   ***************************************************************************/
  public static AssignCallStatement construct
    (LeftValue lval, Name method, ValueExpression[] args)
  {
    return new AssignCallStatement(lval, method, args);
  }
  
  /****************************************************************************
   * Construct a call statement with a result value assigned to a variable.
   * @param lval the left value taking the result
   * @param method the method name
   * @param args the method arguments
   ***************************************************************************/
  protected AssignCallStatement(LeftValue lval, Name method, ValueExpression[] args)
  {
    super(method, args);
    this.lval = lval;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
    lval.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    lval.accept(visitor);
  }
}
