// ----------------------------------------------------------------------------
// BinaryValueExpression.java
// The base class of binary expressions denoting values.
// $Id: BinaryValueExpression.java,v 1.4 2017/07/04 08:04:37 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Program;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * The base class of binary expressions denoting values.
 */
public abstract class BinaryValueExpression extends ValueExpressionBase
{
  public final ValueExpression first;
  public final ValueExpression second;
  
  /****************************************************************************
   * Construct binary expression denoting a value.
   * @param the precedence value of the operator.
   * @param operator the name of the operator.
   * @param first the first (left) argument expression.
   * @param second the second (right) argument expression.
   ***************************************************************************/
  protected BinaryValueExpression(ValueExpression first, ValueExpression second)
  {
    this.first = first;
    this.second = second;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    first.accept(visitor);
    second.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    first.accept(visitor);
    second.accept(visitor);
  }
  
  /***********************************************************************
   * Get textual representation of operator.
   * @return the text.
   **************************************************************************/
  public abstract String getOperator();
}
