// ----------------------------------------------------------------------------
// Handler.java
// An exception handler.
// $Id: Handler.java,v 1.5 2017/07/04 08:04:37 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Program;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * An exception handler.
 */
public final class Handler extends ASTSourceBase
{
  public final Parameter param;
  public final Statement body;
   
  /***************************************************************************
   * Construct an exception handler.
   * @param param the handler parameter
   * @param body the body of the handler
   * @return the handler.
   **************************************************************************/
  public static Handler construct(Parameter param, Statement body)
  {
    return new Handler(param, body);
  }
  
  /***************************************************************************
   * Construct an exception handler.
   * @param param the handler parameter
   * @param body the body of the handler
   **************************************************************************/
  private Handler(Parameter param, Statement body)
  {
    this.param = param;
    this.body = body;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
    param.accept(visitor);
    body.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    param.accept(visitor);
    body.accept(visitor);
  }
}

