// ----------------------------------------------------------------------------
// Strategy.java
// A strategy for solving a task.
// $Id: Strategy.java,v 1.3 2010/03/03 14:01:21 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Tasks;

/**
 * 
 * A strategy for solving a task.
 *
 */
public interface Strategy
{
  /****************************************************************************
   * True if this strategy is applicable to the denoted task.
   * @param task a task
   * @return true iff apply() may be invoked on task.
   ***************************************************************************/
  public boolean isApplicable(Task task);
  
  /****************************************************************************
   * Apply this strategy to a task to which it is applicable.
   * @param task a task to which the task is applicable.
   ***************************************************************************/
  public void apply(Task task);
}
