// ----------------------------------------------------------------------------
// Redo.java
// The "redo" command.
// $Id: Redo.java,v 1.3 2008/10/13 09:32:00 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.ProofNavigator.Commands;

import java.io.PrintWriter;

import fmrisc.ProofNavigator.Proving.*;

/**
 * 
 * The "redo" command: goto an open proof state and redo its proof.
 *
 */
public final class Redo extends ProofCommandBase
{
  public final static String Name = "redo";
  private String label; // the label of the proof state to go to (may be null)
  
  /***************************************************************************
   * create a "redo" command
   * @param label the label of the proof state to go to (may be null)
   **************************************************************************/
  public Redo(String label)
  {
    super(Name);
    this.label = label;
  }
  
  /****************************************************************************
   * Process command "redo": switch to the denoted state (the current state
   * by default) and redo its proof.
   * @param state the state to which the command is applied.
   * @return null
   ***************************************************************************/
  protected ProofState[] processCore(ProofState state)
  {
    Proof proof = state.getProof();
    if (label != null)
    {
      state = proof.getState(label);
      if (state == null)
      {
        error("there is no state with label [" + label + "]");
        return null;
      }
    }
    if (state.isClosed())
    {
      error("cannot redo closed state " + label);
      return null;
    }
    proof.redo(state);
    return null;
  }
  
  /***************************************************************************
   * Prints text representation on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    out.print(getName());
    if (label != null)
    {
      out.print(" ");
      out.print(label);
    }
  }
}
