// ----------------------------------------------------------------------------
// ValueC.java
// The "value" command.
// $Id: ValueC.java,v 1.11 2011/07/18 13:56:15 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.ProofNavigator.Commands;

import java.io.*;

import fmrisc.ProofNavigator.*;
import fmrisc.ProofNavigator.Communication.*;
import fmrisc.ProofNavigator.Semantics.*;
import fmrisc.ProofNavigator.Syntax.*;

/**
 * 
 * Command "print": print value of value identifier.
 *
 */
public final class ValueC extends CommandBase
{
  public final static String Name = "printv";
  private Identifier ident;
  
  /***************************************************************************
   * Create a "value" command.
   * @param ident identifier of denoted value
   **************************************************************************/
  public ValueC(Identifier ident)
  {
    super(Name);
    this.ident = ident;
  }
  
  /****************************************************************************
   * Process command "value": print value of value identifier.
   ***************************************************************************/
  public void process()
  { 
    Environment env = getEnvironment();
    ValueSymbol symbol = env.getValueSymbol(ident);
    if (symbol == null)
    {
      error("there is no value named " + ident.toString() + 
              " visible in current environment");
      return;
    }
    PrintWriter out = State.getSystemOut();
    PrettyPrinter printer = new PrettyPrinter(out);
    ident = symbol.getIdentifier();
    printer.print(ident.toString());
    printer.print(":");
    Type type = symbol.getType();
    printer.print(type);
    Expression value = symbol.getValue();
    if (value != null)
    {
      printer.print(" = ");
      printer.changeLevel(true);
      printer.newLine();
      printer.print(value);
    }
    out.println(".");
  }
  
  /***************************************************************************
   * Prints text representation on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    out.print(getName());
    out.print(" ");
    ident.print(out);
  }
}