// ----------------------------------------------------------------------------
// UnaryExpression.java
// Abstract base class of unary expressions
// $Id: UnaryExpression.java,v 1.13 2008/10/13 09:32:01 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.ProofNavigator.Syntax;
import java.io.*;

/**
 * Abstract base class of unary expressions
 */
public abstract class UnaryExpression extends ExpressionBase
{
  private String op;       // name of operation
  private Expression base; // base expression

  /****************************************************************************
   * constructs binary expression from op and base
   * @param op the name of the operation used for printing
   * @param base the operand
   ***************************************************************************/
  public UnaryExpression(String op, Expression base)
  {
    this.op = op;
    this.base = base;
  }
  
  /***************************************************************************
   * Returns base expression
   * @return the base expression
   **************************************************************************/
  public Expression getBase()
  {
    return base;
  }
  
  /***************************************************************************
   * Prints text representation of tree on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    out.print(op);
    base.printPriority(getPriority(), out);
  }
  
  /***************************************************************************
   * return copy of AST with value references instantiated by the substitutions
   * set in the corresponding declaration identifiers
   * @return the instantiated copy
   **************************************************************************/
  public Expression instantiate()
  {
    return construct(getBase().instantiate());
  }
  
  /***************************************************************************
   * Construct binary expression of the same kind as this expression.
   * @param base an expression.
   * @return a unary expression of the same kind as this expression
   *         with subexpression base.
   **************************************************************************/
  protected abstract UnaryExpression construct(Expression base);
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   * @return the result of the visit (may be null).
   **************************************************************************/
  public AST accept(ASTVisitor visitor) 
  { 
    AST result = visitor.visit(this);
    if (result != null) return result;
    AST base0 = base.accept(visitor);
    if (!visitor.isCloned()) return null;
    return construct((Expression)base0);
  }
}
